""" Using ManagerLive to make batches """
import logging
import os
from datetime import datetime
from pathlib import Path
from typing import Optional

import pandas as pd
from nowcasting_dataset.manager.manager_live import ManagerLive

from nowcasting_forecast import N_GSP

logger = logging.getLogger(__name__)


def make_batches(
    config_filename: str = "nowcasting_forecast/config/mvp_v0.yaml",
    t0_datetime_utc: datetime = None,
    temporary_dir: Optional[str] = None,
    n_gsps: Optional[int] = N_GSP,
    batch_save_dir: Optional[str] = None,
):
    """Make batches from config file"""

    logger.info(f"Making batches using configuration file: {config_filename}")

    if t0_datetime_utc is None:
        t0_datetime_utc = pd.Timestamp(datetime.utcnow()).floor("5T")

    # load config
    manager = ManagerLive()

    manager.load_yaml_configuration(config_filename, set_git=False)

    if temporary_dir is not None:
        manager.config.output_data.filepath = Path(temporary_dir)

    # over write nwp zarr path
    if (os.getenv("NWP_PATH", None) is not None) and (manager.config.input_data.nwp is not None):
        manager.config.input_data.nwp.nwp_zarr_path = os.getenv("NWP_PATH")
        logger.debug(f"WIll be opening nwp file: {manager.config.input_data.nwp.nwp_zarr_path}")

    # over write hrv sat zarr path
    if (os.getenv("HRV_SAT_PATH", None) is not None) and (
        manager.config.input_data.hrvsatellite is not None
    ):
        manager.config.input_data.hrvsatellite.hrvsatellite_zarr_path = os.getenv("HRV_SAT_PATH")
        logger.debug(
            f"WIll be opening sat file:"
            f" {manager.config.input_data.hrvsatellite.hrvsatellite_zarr_path}"
        )

    # over write sat zarr path
    if (os.getenv("SAT_PATH", None) is not None) and (
        manager.config.input_data.satellite is not None
    ):
        manager.config.input_data.satellite.satellite_zarr_path = os.getenv("SAT_PATH")
        logger.debug(
            f"WIll be opening sat file: {manager.config.input_data.satellite.satellite_zarr_path}"
        )

    # make location file
    manager.initialize_data_sources(
        names_of_selected_data_sources=["gsp", "nwp", "pv", "satellite", "hrvsatellite", "sun"]
    )
    manager.create_files_specifying_spatial_and_temporal_locations_of_each_example(
        t0_datetime=t0_datetime_utc,
        n_gsps=n_gsps,
    )

    # remove gsp as a datasource
    if not manager.config.input_data.gsp.is_live:
        manager.data_sources.pop("gsp")

    # make batches
    manager.create_batches()

    # save batch to s3, just save batch 0
    if batch_save_dir is not None:
        manager.save_batch(batch_idx=0, path=batch_save_dir)



import os
import tempfile

import pytest
import xarray as xr
import zarr
from click.testing import CliRunner
from nowcasting_datamodel.connection import DatabaseConnection
from nowcasting_datamodel.models import Forecast, ForecastSQL, LocationSQL
from nowcasting_datamodel.fake import make_fake_me_latest

from nowcasting_forecast import N_GSP
from nowcasting_forecast.app import run


def test_fake(db_connection, me_latest):
    runner = CliRunner()
    response = runner.invoke(
        run,
        ["--db-url", db_connection.url, "--fake", "true", "--n-gsps", "10"],
        catch_exceptions=True,
    )
    if response.exception:
        raise response.exception
    assert response.exit_code == 0

    with db_connection.get_session() as session:
        forecasts = session.query(ForecastSQL).all()
        _ = Forecast.from_orm(forecasts[0])
        assert len(forecasts) == (10 + 1) * 2  # 10 gsp + national, x2 for historic ones too

        locations = session.query(LocationSQL).all()
        assert len(locations) == 10 + 1


def test_fake_twice(db_connection: DatabaseConnection, me_latest):
    runner = CliRunner()
    response = runner.invoke(
        run,
        ["--db-url", db_connection.url, "--fake", "true", "--n-gsps", "10"],
        catch_exceptions=True,
    )
    if response.exception:
        raise response.exception

    assert response.exit_code == 0, response.exception

    with db_connection.get_session() as session:
        forecasts = session.query(ForecastSQL).all()
        _ = Forecast.from_orm(forecasts[0])

        assert len(forecasts) == (10 + 1) * 2  # 338 gsp + national, x2 for historic ones too

        locations = session.query(LocationSQL).all()
        assert len(locations) == 10 + 1

    response = runner.invoke(
        run, ["--db-url", db_connection.url, "--fake", "true", "--n-gsps", "10"]
    )
    assert response.exit_code == 0

    with db_connection.get_session() as session:
        forecasts = session.query(ForecastSQL).all()
        _ = Forecast.from_orm(forecasts[0])
        assert len(forecasts) == (10 + 1) * 3  # 338 gsp + national

        locations = session.query(LocationSQL).all()
        assert len(locations) == 10 + 1


def test_not_fake(
    db_connection: DatabaseConnection,
    nwp_data: xr.Dataset,
    input_data_last_updated,
    sat_data,
    hrv_sat_data,
    gsp_yields_and_systems,
    pv_yields_and_systems,
    me_latest,
):
    with tempfile.TemporaryDirectory() as temp_dir:
        # save nwp data
        nwp_path = f"{temp_dir}/unittest.netcdf"
        nwp_data.to_netcdf(nwp_path, engine="h5netcdf")
        os.environ["NWP_PATH"] = nwp_path
        hrv_sat_path = f"{temp_dir}/hrv_sat_unittest.zarr.zip"
        with zarr.ZipStore(hrv_sat_path) as store:
            hrv_sat_data.to_zarr(store, compute=True)
        os.environ["HRV_SAT_PATH"] = hrv_sat_path
        sat_path = f"{temp_dir}/sat_unittest.zarr.zip"
        with zarr.ZipStore(sat_path) as store:
            sat_data.to_zarr(store, compute=True)
        os.environ["SAT_PATH"] = sat_path

        runner = CliRunner()
        response = runner.invoke(
            run, ["--db-url", db_connection.url, "--fake", "false", "--n-gsps", "10"]
        )
        assert response.exit_code == 0, response

        with db_connection.get_session() as session:
            forecasts = session.query(ForecastSQL).all()
            _ = Forecast.from_orm(forecasts[0])
            assert len(forecasts) == (10 + 1) * 2  # 317 gsp + national, x2 for historic ones too
            assert len(forecasts[0].forecast_values) > 1


@pytest.mark.skip("CI doesnt have access to AWS for model weights")
def test_mwp_1(db_connection: DatabaseConnection, nwp_data: xr.Dataset, input_data_last_updated):
    with tempfile.TemporaryDirectory() as temp_dir:
        # save nwp data
        nwp_path = f"{temp_dir}/unittest.netcdf"
        nwp_data.to_netcdf(nwp_path, engine="h5netcdf")
        os.environ["NWP_PATH"] = nwp_path

        runner = CliRunner()
        response = runner.invoke(
            run,
            [
                "--db-url",
                db_connection.url,
                "--fake",
                "false",
                "--model-name",
                "nwp_simple_trained",
                "--n-gsps",
                "10",
            ],
        )
        assert response.exit_code == 0, response

        with db_connection.get_session() as session:
            forecasts = session.query(ForecastSQL).all()
            _ = Forecast.from_orm(forecasts[0])
            assert len(forecasts) == 10 + 1  # 10 gsp + national
            assert len(forecasts[0].forecast_values) > 1



""" Simple model to take NWP irradence and make solar """
import logging
from datetime import timedelta, timezone
from typing import Optional, Union

import pandas as pd
import xarray as xr
from nowcasting_dataset.dataset.batch import Batch

logger = logging.getLogger(__name__)

NAME = "nwp_simple"


def nwp_irradiance_simple_run_one_batch(
    batch: Union[dict, Batch],
    n_examples: Optional[int] = None,
) -> pd.DataFrame:
    """Run model for one batch"""

    # make sure its a Batch object
    if type(batch) == dict:
        batch = Batch(**batch)

    if n_examples is None:
        n_examples = batch.metadata.batch_size

    # run model
    irradiance_mean = nwp_irradiance_simple(batch)

    forecasts = []
    for i in range(batch.metadata.batch_size):
        if i >= n_examples:
            break

        # get id from location
        gsp_id = batch.metadata.space_time_locations[i].id

        # t0 value value
        t0_datetime_utc = batch.metadata.space_time_locations[i].t0_datetime_utc.replace(
            tzinfo=timezone.utc
        )

        for t_index in irradiance_mean.time_index:
            target_time = t0_datetime_utc + timedelta(minutes=30) * t_index.values

            forecasts.append(
                dict(
                    t0_datetime_utc=t0_datetime_utc,
                    target_datetime_utc=target_time,
                    forecast_gsp_pv_outturn_mw=float(irradiance_mean[i, t_index].values[0]),
                    gsp_id=gsp_id,
                )
            )

    forecasts = pd.DataFrame(forecasts)

    return forecasts


def nwp_irradiance_simple(batch: Batch) -> xr.DataArray:
    """Predictions for one batch

    Just to take the mean NWP data across the batch

    Returns a data array of nwp irradiance with dimensions for examples and time.
    """
    nwp = batch.nwp

    # take solar irradence # TODO
    # print(nwp)
    # nwp.data = nwp.data.sel(channels_index=["dlwrf"])

    # take mean across all x and y dims

    irradence_mean = nwp.data.mean(axis=(2, 3))

    # scale irradence to roughly mw
    irradence_mean = irradence_mean / 10

    return irradence_mean
